package com.example.flutter_gaode_map;

import android.app.Activity;
import android.app.Application;
import android.os.Bundle;
import android.util.Log;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationListener;
import com.example.flutter_gaode_map.mapview.MapviewFactory;
import com.example.flutter_gaode_map.textview.MyViewFactory;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** FlutterGaodeMapPlugin */
public class FlutterGaodeMapPlugin implements  MethodChannel.MethodCallHandler, EventChannel.StreamHandler{
  private static final String TAG = "AmapLocationPlugin";
  private String mLocation;
  private EventChannel.EventSink mEventSink;
  //声明AMapLocationClient类对象
  private AMapLocationClient mLocationClient = null;
  public static PluginRegistry.Registrar registrar;
  private static int registrarActivityHashCode = 0;
  private static AtomicInteger activityState= new AtomicInteger(0);
  public static final int CREATED = 1;
  public static final int RESUMED = 3;
  public static final int STOPPED = 5;
  public static final int DESTROYED = 6;
  //异步获取定位结果
  private AMapLocationListener mAMapLocationListener = new AMapLocationListener() {
    @Override
    public void onLocationChanged(AMapLocation amapLocation) {
      if (amapLocation != null) {
        if (amapLocation.getErrorCode() == 0) {

          mLocation = getLocationInfoMap(amapLocation);
          mEventSink.success(mLocation);
          Log.d("onLocationChanged", mLocation);
        }
      }
    }
  };

  private String getLocationInfoMap(AMapLocation amapLocation) {
    Map<String, String> map = new HashMap<>();
    map.put("longitude", String.valueOf(amapLocation.getLongitude()));
    map.put("latitude", String.valueOf(amapLocation.getLatitude()));
    map.put("province", amapLocation.getProvince());
    map.put("city", amapLocation.getCity());
    map.put("district", amapLocation.getDistrict());
    map.put("address", amapLocation.getAddress());
    return new JSONObject(map).toString();
  }

  /**
   * Plugin registration.
   */
  public static void registerWith(Registrar registrar) {
    FlutterGaodeMapPlugin.registrar = registrar;
    registrarActivityHashCode = registrar.activity().hashCode();

    // 注册生命周期回调, 保证地图初始化的时候对应的是正确的activity状态
    registrar.activity().getApplication().registerActivityLifecycleCallbacks(new Application.ActivityLifecycleCallbacks() {
      @Override
      public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
        if (activity.hashCode() != registrarActivityHashCode) {
          return;
        }
        activityState.set(CREATED);
      }

      @Override
      public void onActivityStarted(Activity activity) {
        if (activity.hashCode() != registrarActivityHashCode) {
          return;
        }
      }

      @Override
      public void onActivityResumed(Activity activity) {
        if (activity.hashCode() != registrarActivityHashCode) {
          return;
        }
        activityState.set(RESUMED);
      }

      @Override
      public void onActivityPaused(Activity activity) {
        if (activity.hashCode() != registrarActivityHashCode) {
          return;
        }
      }

      @Override
      public void onActivityStopped(Activity activity) {
        if (activity.hashCode() != registrarActivityHashCode) {
          return;
        }
        activityState.set(STOPPED);
      }

      @Override
      public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

      }

      @Override
      public void onActivityDestroyed(Activity activity) {
        if (activity.hashCode() != registrarActivityHashCode) {
          return;
        }
        activityState.set(DESTROYED);
      }
    });
    final MethodChannel methodChannel = new MethodChannel(registrar.messenger(), "plugin.kinsomy.com/methodchannel");
    final EventChannel eventChannel = new EventChannel(registrar.messenger(), "plugin.kinsomy.com/eventchannel");
    final MethodChannel keyChannel = new MethodChannel(registrar.messenger(), "me.yohom/amap_base");
    final FlutterGaodeMapPlugin instance = new FlutterGaodeMapPlugin(registrar);
    methodChannel.setMethodCallHandler(instance);
    keyChannel.setMethodCallHandler(instance);
    eventChannel.setStreamHandler(instance);
    registrar
            .platformViewRegistry()
            .registerViewFactory(
                    "plugins.felix.angelov/textview", new MyViewFactory(registrar.messenger()));
    registrar.platformViewRegistry().registerViewFactory("plugins.nightfarmer.top/myview", new MapviewFactory(registrar.messenger(),activityState));

  }

  FlutterGaodeMapPlugin(Registrar registrar) {
    this.registrar = registrar;
    //初始化定位
    mLocationClient = new AMapLocationClient(registrar.context());

    //设置定位回调监听
    mLocationClient.setLocationListener(mAMapLocationListener);
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("startLocation")) {
      //启动定位
      mLocationClient.startLocation();
    } else if (call.method.equals("stopLocation")) {
      //停止定位
      mLocationClient.stopLocation();
    } else if (call.method.equals("getLocation")) {
      result.success(mLocation);
    } else if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }else if (call.method.equals("setKey")) {
      Log.e("-------------setkey",call.argument("key").toString());
      result.success("android端需要在Manifest里配置key ");
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onListen(Object o, EventChannel.EventSink eventSink) {
    this.mEventSink = eventSink;
  }

  @Override
  public void onCancel(Object o) {
    mLocationClient.stopLocation();
  }


}
