import 'dart:async';
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

class FlutterGaodeMap {

  static final _channel = MethodChannel('me.yohom/amap_base');

  static Map<String, List<String>> assetManifest;

  static Future init(String key) async {
    _channel.invokeMethod('setKey', {'key': key});

    // 加载asset相关信息, 供区分图片分辨率用, 因为native端的加载asset方法无法区分分辨率, 这是一个变通方法
    assetManifest =
    await rootBundle.loadStructuredData<Map<String, List<String>>>(
      'AssetManifest.json',
          (String jsonData) {
        if (jsonData == null)
          return SynchronousFuture<Map<String, List<String>>>(null);

        final Map<String, dynamic> parsedJson = json.decode(jsonData);
        final Iterable<String> keys = parsedJson.keys;
        final Map parsedManifest = Map<String, List<String>>.fromIterables(
        keys,
        keys.map<List<String>>((key) => List<String>.from(parsedJson[key])),
        );
        return SynchronousFuture<Map<String, List<String>>>(parsedManifest);
      },
    );

//    await AMapLocation().init();
  }

  @Deprecated('使用init方法初始化的时候设置key')
  static Future setKey(String key) {
    return _channel.invokeMethod('setKey', {'key': key});
  }




  factory FlutterGaodeMap() {
    if (_instance == null) {
      final MethodChannel methodChannel =
      const MethodChannel('plugin.kinsomy.com/methodchannel');
      final EventChannel eventChannel =
      const EventChannel('plugin.kinsomy.com/eventchannel');
      final MethodChannel _keyChannel = MethodChannel(
          'me.yohom/amap_base');
      _instance =
          FlutterGaodeMap.private(methodChannel, eventChannel);
    }
    return _instance;
  }

  FlutterGaodeMap.private(this._methodChannel, this._eventChannel);

  static FlutterGaodeMap _instance;

  final MethodChannel _methodChannel;
  final EventChannel _eventChannel;
  Stream<String> _onLocationFetched;

  /// Fires whenever the location send.
  Stream<String> get onLocationChanged {
    if (_onLocationFetched == null) {
      _onLocationFetched =
          _eventChannel.receiveBroadcastStream().map((dynamic event) => event);
    }
    return _onLocationFetched;
  }

  Future<void> get startLocation => _methodChannel.invokeMethod("startLocation");

  Future<void> get stopLocation => _methodChannel.invokeMethod("stopLocation");

  Future<String> get getLocation => _methodChannel
      .invokeMethod("getLocation")
      .then<String>((dynamic result) => result);


}
